package com.four.auth.controller;

import javax.servlet.http.HttpServletRequest;
import com.four.common.duck.request.RegistrationInformationRequest;
import com.four.auth.service.SysMailService;
import com.four.common.duck.request.RequestRegistrationInformation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.four.auth.form.LoginBody;
import com.four.auth.form.RegisterBody;
import com.four.auth.service.SysLoginService;
import com.four.common.core.domain.R;
import com.four.common.core.utils.JwtUtils;
import com.four.common.core.utils.StringUtils;
import com.four.common.security.auth.AuthUtil;
import com.four.common.security.service.TokenService;
import com.four.common.security.utils.SecurityUtils;
import com.four.system.api.model.LoginUser;

/**
 * token 控制
 *
 * @author ruoyi
 */
@RestController
@SuppressWarnings("all")
public class TokenController {

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysLoginService sysLoginService;

    @Autowired
    private SysMailService sysMailService;

    /***
     * 邮箱密码登录
     * @param form
     * @return
     */
    @PostMapping("login")
    public R<?> login(@RequestBody LoginBody form) {
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getEmail(), form.getPassword());
        // 获取登录token
        return R.ok(tokenService.createToken(userInfo));
    }


    /**
     * 人脸邮箱登录
     * @param form
     * @return
     */
    @PostMapping("emailLogin")
    public R<?> emailLogin(@RequestBody LoginBody form){

        LoginUser loginUser = sysLoginService.emailLogin(form.getEmail());

        return R.ok(tokenService.createToken(loginUser));
    }
    /**
     * 用户名密码登录
     * @param form
     * @return
     */
    @PostMapping("userLogin")
    public R<?> userLogin(@RequestBody LoginBody form) {

        LoginUser loginUser = sysLoginService.userLogin(form.getUsername(), form.getPassword());

        return R.ok(tokenService.createToken(loginUser));
    }

    /***
     * 登出
     * @param request
     * @return
     */
    @DeleteMapping("logout")
    public R<?> logout(HttpServletRequest request) {
        String token = SecurityUtils.getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            String username = JwtUtils.getUserName(token);
            // 删除用户缓存记录
            AuthUtil.logoutByToken(token);
            // 记录用户退出日志
            sysLoginService.logout(username);
        }
        return R.ok();
    }

    @PostMapping("refresh")
    public R<?> refresh(HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser)) {
            // 刷新令牌有效期
            tokenService.refreshToken(loginUser);
            return R.ok();
        }
        return R.ok();
    }


    @PostMapping("register")
    public R<?> register(@RequestBody RegisterBody registerBody) {
        // 用户注册
        sysLoginService.register(registerBody.getUsername(), registerBody.getPassword());
        return R.ok();
    }

    /**
     * 医生注册
     * @param healthRegisterBody
     * @return
     */
    @PostMapping("healthRegisterBody")
    public R<?> healthRegisterBody(@RequestBody RequestRegistrationInformation requestRegistrationInformation) {

        sysMailService.verifyCode(requestRegistrationInformation.getEmail(), requestRegistrationInformation.getCode());

        sysLoginService.healthRegister(requestRegistrationInformation);
        return R.ok();
    }

    /***
     * 患者注册
     */
    @PostMapping("PatientRegisterBody")
    public R<?> PatientRegisterBody(@RequestBody RegisterBody registerBody){
        sysMailService.verifyCode(registerBody.getEmail(),registerBody.getCode());

        sysLoginService.PatientRegisterBody(registerBody);
        return R.ok();
    }


    /***
     * 发送验证码
     * @param email
     * @return
     */
    @GetMapping("/sendCode/{email}")
    public R<?> sendCode(@PathVariable String email) {
        sysMailService.sendCode(email);
        return R.ok();
    }


}
