package com.four.auth.service;

import com.four.common.core.constant.SecurityConstants;
import com.four.common.core.domain.R;
import com.four.common.core.exception.ServiceException;
import com.four.common.core.utils.StringUtils;
import com.four.common.redis.service.RedisService;
import com.four.system.api.RemoteUserService;
import com.four.system.api.model.LoginUser;

import org.apache.commons.lang.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Component
public class SysMailService {

    @Autowired
    private RemoteUserService remoteUserService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private JavaMailSender javaMailSender;
    public void verifyCode(String email, String code) {
        if(StringUtils.isAnyBlank(email) || StringUtils.isAllBlank(code)){
            throw new ServiceException("邮箱验证码不能为空");
        }

        //判断缓存中是否存在验证码
        String verifyCode = redisService.getCacheObject(email);
        if(StringUtils.isAllBlank(verifyCode)){
            throw new ServiceException("请先获取验证码");
        }

        //验证码校验
        if(!code.equals(verifyCode)){
            throw new ServiceException("验证码错误");
        }
    }

    public void sendCode(String email) {
        if(StringUtils.isAllBlank(email)){
            throw new ServiceException("邮箱为空");
        }

        String verifyCode = redisService.getCacheObject(email);
        if(!StringUtils.isAnyBlank(verifyCode)){
            throw new ServiceException("验证码已发送,请稍后重试");
        }


        R<LoginUser> userInfoByEmail = remoteUserService.getUserInfoByEmail(email, SecurityConstants.INNER);
        if(!StringUtils.isNull(userInfoByEmail.getData())){
            throw new ServiceException("该邮箱:" + email + "已被注册");
        }


        verifyCode = RandomStringUtils.randomNumeric(4);

        MimeMessage mimeMessage = javaMailSender.createMimeMessage();
        try {
            MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
            helper.setFrom("1733802689@qq.com");
            helper.setText("验证码为:"+verifyCode);
            helper.setTo(email);
            helper.setSubject("我来发邮件来了");
            javaMailSender.send(mimeMessage);
        } catch (MessagingException e) {
            throw new ServiceException("邮箱服务出错了:{}" + e.getMessage());
        }

        redisService.setCacheObject(email,verifyCode,5L, TimeUnit.MINUTES);
    }

}
